@@ -7,11 +7,11 @@ def download_file_from_google_drive(fid, destination):
 
     session = requests.Session()
 
-    response = session.get(url, params={'id': fid}, stream=True)
+    response = session.get(url, params={"id": fid}, stream=True)
     token = get_confirm_token(response)
 
     if token:
-        params = {'id': fid, 'confirm': token}
+        params = {"id": fid, "confirm": token}
         response = session.get(url, params=params, stream=True)
 
     save_response_content(response, destination)
@@ -19,7 +19,7 @@ def download_file_from_google_drive(fid, destination):
 
 def get_confirm_token(response):
     for key, value in response.cookies.items():
-        if key.startswith('download_warning'):
+        if key.startswith("download_warning"):
             return value
 
     return None
@@ -30,13 +30,13 @@ def save_response_content(response, destination):
 
     with open(destination, "wb") as f:
         for chunk in response.iter_content(chunk_size):
-            if chunk: # filter out keep-alive new chunks
+            if chunk:  # filter out keep-alive new chunks
                 f.write(chunk)
 
 
 if __name__ == "__main__":
-    file_id = '1JJoeO9FQ9vAdmhALma7wpgqs3OTTPPwF'
-    destination = 'test.tar.gz'
+    file_id = "1JJoeO9FQ9vAdmhALma7wpgqs3OTTPPwF"
+    destination = "test.tar.gz"
     download_file_from_google_drive(file_id, destination)
     # Unzip data
     tar = tarfile.open(destination, "r:gz")
